@@ -167,45 +167,7 @@ def_sXaug_Matrix_QRP_CHM_scaled <- function(Xaug,weight_X,w.ranef,H_global_scale
     if (is.null(BLOB$logdet_R_scaled_v)) BLOB$logdet_R_scaled_v <- Matrix::determinant(BLOB$CHMfactor_wd2hdv2w)$modulus[1]
     return(BLOB$logdet_R_scaled_v)
   } else if (which=="beta_cov_info_from_sXaug") {
-    if (TRUE) {
-      tcrossfac_v_beta_cov <- solve(BLOB$R_scaled) # *M*atrix
-      tPmat <- sparseMatrix(seq_along(BLOB$sortPerm), BLOB$sortPerm, x=1)
-      pforpv <- attr(sXaug,"pforpv")
-      X_scaling <- sqrt(rep(attr(sXaug,"H_global_scale"),pforpv))
-      if ( ! is.null(B)) X_scaling <- X_scaling/B
-      diagscalings <- c(1/sqrt(attr(sXaug,"w.ranef")), X_scaling)
-      tPmat <- .Dvec_times_Matrix(diagscalings, tPmat) # Pmat <- .Matrix_times_Dvec(Pmat,diagscalings)
-      tcrossfac_v_beta_cov <- as.matrix(tPmat %*% tcrossfac_v_beta_cov)
-      rownames(tcrossfac_v_beta_cov) <- colnames(sXaug) ## necessary for summary.HLfit, already lost in BLOB$R_scaled
-      seqp <- seq_len(pforpv)
-      beta_pos <- attr(sXaug,"n_u_h")+seqp
-      beta_v_order <- c(beta_pos,seq(attr(sXaug,"n_u_h")))
-      tcrossfac_beta_v_cov <- tcrossfac_v_beta_cov[beta_v_order,,drop=FALSE]
-      beta_cov <- .tcrossprod(tcrossfac_beta_v_cov[seqp,,drop=FALSE])
-      return( list(beta_cov=beta_cov, 
-                   #beta_v_cov=beta_v_cov,
-                   tcrossfac_beta_v_cov=tcrossfac_beta_v_cov) )
-    }
-    ########################
-    if (FALSE) {
-      ## First version quite slow (test-negbin1): chol2inv, [BLOB$sortPerm,BLOB$sortPerm] on spare matrix, .Dvec_times_matrix 
-      beta_cov <- Matrix::chol2inv(BLOB$R_scaled) ## actually augmented v_beta_cov as the following shows
-      pforpv <- attr(sXaug,"pforpv")
-      v_beta_cov <- as.matrix(beta_cov[BLOB$sortPerm,BLOB$sortPerm])
-      diagscalings <- sqrt(c(1/attr(sXaug,"w.ranef"),rep(attr(sXaug,"H_global_scale"),pforpv)))
-      v_beta_cov <- .Dvec_times_matrix(diagscalings, .m_Matrix_times_Dvec(v_beta_cov, diagscalings))
-    } else {
-      ## Second version faster, but could we avoid as.matrix ? (and t() ?). If not as.matrix, avoid [.,.] ?
-      Pmat <- sparseMatrix(seq_along(BLOB$perm), BLOB$perm, x=1)
-      pforpv <- attr(sXaug,"pforpv")
-      diagscalings <- sqrt(c(1/attr(sXaug,"w.ranef"),rep(attr(sXaug,"H_global_scale"),pforpv)))
-      Pmat <- .Matrix_times_Dvec(Pmat,diagscalings)
-      v_beta_cov <- as.matrix(crossprod(solve(t(BLOB$R_scaled),Pmat)))
-    }
-    colnames(v_beta_cov) <- colnames(sXaug) ## necessary for summary.HLfit
-    beta_pos <- attr(sXaug,"n_u_h")+seq_len(pforpv)
-    beta_v_order <- c(beta_pos,seq(attr(sXaug,"n_u_h"))) ## 
-    return( list(beta_cov=v_beta_cov[beta_pos,beta_pos,drop=FALSE], beta_v_cov=v_beta_cov[beta_v_order,beta_v_order]) )
+    return(.calc_beta_cov_info_from_sXaug(BLOB=BLOB, sXaug=sXaug, B=B))
   } else if (which=="beta_cov_info_from_wAugX") { ## using a weighted Henderson's augmented design matrix, not a true sXaug  
     if (TRUE) {
       tcrossfac_beta_v_cov <- solve(BLOB$R_scaled)
