@@ -1,6 +1,6 @@
 cat("\nTest of adjacency (long fits):")
 
-if (spaMM.getOption("example_maxtime")>360) { # 
+if (spaMM.getOption("example_maxtime")>227) { # 
   ## fixme This is sparse_precision LevM: improve efficiency? 
   ## example suggested by Jeroen van den Ochtend jeroenvdochtend@gmail.com Jeroen.vandenochtend@business.uzh.ch
   ## it's no use to try sparse_precision=FALSE bc bc the augmented sigma matrix is huge
@@ -14,8 +14,8 @@ if (spaMM.getOption("example_maxtime")>360) { #
                           control.HLfit=list(LevenbergM=FALSE), ## inhibits default for binary data 
                           verbose=c(TRACE=interactive()), # to trace convergence 
                           adjMatrix=adjlgMat
-    ) ## F I X M E refitting lambda (by request) gives a lower lik...
-  }) ##  ~107 (v.2.4.102)  ## 102 in v.2.5.34 ## 99.99 in v2.6.53
+    ) ## F I X M E refitting lambda (by request) gives a lower lik... (-1552.946 v2.7.19)
+  }) ##  ~107 (v.2.4.102)  ## 102 in v.2.5.34 ## 99.99 in v2.6.53 # 64.43utilin v2.7.1 # 55.12util in v2.7.6 #53.49util in v2.7.27  
   expectedMethod <- "AUGI0_ZX_sparsePrecision" 
   if (interactive()) {
     if (! (expectedMethod %in% IRLS.Frailty$MME_method)) {
@@ -29,7 +29,7 @@ if (spaMM.getOption("example_maxtime")>360) { #
                           #fixed=list(rho = -0.0294184,  lambda = 0.241825),
                           adjMatrix=adjlgMat
     )
-  }) ## ~359 (v.2.4.102) => 250 in v.2.5.34; 242.44 in v2.6.53
+  }) ## ~359 (v.2.4.102) => 250 in v.2.5.34; 242.44 in v2.6.53 # 156.44util in v2.7.1 # 152.23util in v2.7.6 # 146.38util in v.2.7.27
   # spprec_LevM_D=="colSums" gives the highest lik, but "1" is fastest; "rowSums" may be slowest.
   spaMM.options(oldop)
 }
\ No newline at end of file
